@@ -24,6 +24,7 @@
 ## 2018-06-01 esa.plot type replaces as.density
 ##            esa.plot conditional argument
 ##            esa.plot CVpdot
+## 2019-01-21 poly.habitat argument in pdot.contour and buffer.contour functions etc.
 ###############################################################################
 
 pdot <- function (X, traps, detectfn = 0, detectpar = list(g0 = 0.2, sigma = 25, z = 1),
@@ -200,12 +201,13 @@ hdot <- function (X, traps, detectfn = 14, detectpar = list(lambda0 = 0.2, sigma
 ############################################################################################
 
 esa.plot <- function (object, max.buffer = NULL, spacing = NULL, max.mask = NULL, detectfn,
-                      detectpar, noccasions, binomN = NULL, thin = 0.1, poly = NULL, session = 1,
+                      detectpar, noccasions, binomN = NULL, thin = 0.1, poly = NULL, 
+                      poly.habitat = TRUE, session = 1,
                       plt = TRUE, type = c('density', 'esa','meanpdot', 'CVpdot'), 
                       n = 1, add = FALSE, overlay = TRUE, conditional = FALSE, ...) {
     type <- match.arg(type)
     if (inherits(object, 'secr')) {
-        esa.plot.secr (object, max.buffer, max.mask, thin, poly, session, plt,
+        esa.plot.secr (object, max.buffer, max.mask, thin, poly, poly.habitat, session, plt,
                        type, add, overlay, conditional, ...)
     }
     else {
@@ -213,7 +215,7 @@ esa.plot <- function (object, max.buffer = NULL, spacing = NULL, max.mask = NULL
         if (inherits(object, 'secrlist')) {
             output <- vector('list')
             arg <- list(max.buffer = max.buffer, max.mask = max.mask, thin = thin, 
-                        poly = poly, session = session, plt = plt, type = type, 
+                        poly = poly, poly.habitat = poly.habitat, session = session, plt = plt, type = type, 
                         add = add, conditional = conditional)
             extra <- list(...)
             if (!('col' %in% names(extra)))
@@ -243,7 +245,7 @@ esa.plot <- function (object, max.buffer = NULL, spacing = NULL, max.mask = NULL
             if(is.null(max.mask)) {
                 if (is.null(spacing))
                     spacing <- spacing(object)/3
-                max.mask <- make.mask (object, max.buffer, spacing,,, 'trapbuffer', poly)
+                max.mask <- make.mask (object, max.buffer, spacing,,, 'trapbuffer', poly, poly.habitat)
             }
             nmask <- nrow(max.mask)
             detectfn <- valid.detectfn(detectfn)
@@ -331,7 +333,7 @@ esa.plot <- function (object, max.buffer = NULL, spacing = NULL, max.mask = NULL
 ###############################################################################
 
 esa.plot.secr <- function (object, max.buffer = NULL, max.mask = NULL,
-    thin = 0.1, poly = NULL, session = 1, plt = TRUE, type = 'density',
+    thin = 0.1, poly = NULL, poly.habitat = TRUE, session = 1, plt = TRUE, type = 'density',
     add = FALSE, overlay = TRUE, conditional = FALSE, ...) {
 
     if (!inherits(object,'secr'))
@@ -353,7 +355,7 @@ esa.plot.secr <- function (object, max.buffer = NULL, max.mask = NULL,
             addthisone <- ifelse (add | (overlay & (i != session[1])),
                                   TRUE, FALSE)
             esa.plot.outputs[[i]] <- esa.plot.secr (object, max.buffer,
-                max.mask, thin, poly, i, plt, type, addthisone,
+                max.mask, thin, poly, poly.habitat, i, plt, type, addthisone,
                 overlay, conditional, ...)
         }
         if (plt)
@@ -391,7 +393,7 @@ esa.plot.secr <- function (object, max.buffer = NULL, max.mask = NULL,
         }
         binomN <- object$details$binomN
         esa.plot (trps, max.buffer, spacg, max.mask, object$detectfn, detpar,
-                  nocc, binomN, thin, poly, session, plt, type, n, add, overlay, 
+                  nocc, binomN, thin, poly, poly.habitat, session, plt, type, n, add, overlay, 
                   conditional, ...)
     }
 }
@@ -404,14 +406,14 @@ pdot.contour <- function (traps, border = NULL, nx = 64, detectfn = 0,
 ## no means of passing mask covariates...
                             noccasions = NULL, binomN = NULL,
                           levels = seq(0.1, 0.9, 0.1),
-                          poly = NULL, plt = TRUE, add = FALSE, fill = NULL, ...) {
+                          poly = NULL, poly.habitat = TRUE, plt = TRUE, add = FALSE, fill = NULL, ...) {
     if (ms(traps)) {
         if (length(noccasions) == 1)
             noccasions <- rep(noccasions,length(traps))
         output <- mapply(pdot.contour, traps, detectpar=detectpar, noccasions=noccasions,
                          MoreArgs = list(border = border, nx = nx,
                          detectfn = detectfn, binomN = binomN,
-                         levels = levels, poly = poly, plt = plt, add = add, ...))
+                         levels = levels, poly = poly, poly.habitat = poly.habitat, plt = plt, add = add, ...))
         if (plt)
             invisible(output)
         else
@@ -427,7 +429,10 @@ pdot.contour <- function (traps, border = NULL, nx = 64, detectfn = 0,
         z <- pdot(tempmask, traps, detectfn, detectpar, noccasions, binomN)
         if (!is.null(poly)) {
             OK <- pointsInPolygon(tempmask, poly)
-            z[!OK] <- 0
+            if (poly.habitat)
+                z[!OK] <- 0
+            else
+                z[OK] <- 0
         }
         if (plt) {
             contour (xlevels, ylevels, matrix(z, nrow = nx), add = add, levels = levels, ...)
@@ -451,7 +456,8 @@ pdot.contour <- function (traps, border = NULL, nx = 64, detectfn = 0,
 ############################################################################################
 
 buffer.contour <- function (traps, buffer, nx = 64, convex = FALSE, ntheta = 100,
-                            plt = TRUE, add = FALSE, poly = NULL, fill = NULL, ...) {
+                            plt = TRUE, add = FALSE, poly = NULL, poly.habitat = TRUE, 
+                            fill = NULL, ...) {
     oneconvexbuffer <- function (buffer) {
         temp  <- data.frame(x = apply(expand.grid(traps$x, buffer * cos(theta)),1,sum),
                        y = apply(expand.grid(traps$y, buffer * sin(theta)),1,sum))
@@ -467,7 +473,7 @@ buffer.contour <- function (traps, buffer, nx = 64, convex = FALSE, ntheta = 100
 
     if (ms(traps)) {
         output <- lapply(traps, buffer.contour, buffer = buffer, nx = nx, convex = convex,
-               ntheta = ntheta, plt = plt, add = add, poly = poly, ...)
+               ntheta = ntheta, plt = plt, add = add, poly = poly, poly.habitat = poly.habitat, ...)
         if (plt)
             invisible(output)
         else
@@ -494,7 +500,10 @@ buffer.contour <- function (traps, buffer, nx = 64, convex = FALSE, ntheta = 100
             z <- distancetotrap(tempmask, traps)
             if (!is.null(poly)) {
                 OK <- pointsInPolygon(tempmask, poly)
-                z[!OK] <- 1e20
+                if (poly.habitat)
+                    z[!OK] <- 1e20
+                else
+                    z[OK] <- 0
             }
             if (plt) {
                 contour (xlevels, ylevels, matrix(z, nrow = nx), add = add,
